/*****************************************************
Chip type           : ATmega16
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 256
*****************************************************/

#include <mega16.h> 
#include <delay.h>


unsigned int tick = 0;          //  
unsigned char b_cnt = 0;        //  
bit start_cond = 0;             //  
bit repeat_cond = 0;            // 

unsigned char addr_1 = 0;       //  
unsigned char addr_0 = 0;       //  
unsigned char cmd_1 = 0;        //  
unsigned char cmd_0 = 0;        //  
unsigned char addr = 0;         // 
unsigned char cmd = 0;          // 

// External Interrupt 0 service routine
interrupt [EXT_INT0] void ext_int0_isr(void)
        {
        TCNT0=0xFD;                                                             // 
        TCCR0=0x04;
        
        if (tick >= 139 && tick < 150)                                          //   139  150 
                {
                start_cond = 1;                                                 //  
                repeat_cond = 0;                                                //  
                addr_1 = addr_0 = cmd_1 = cmd_0 = 0;                            //     
                }
        if (tick >= 116 && tick < 139)                                          //   116  138 
                {
                repeat_cond = 1;                                                //  
                start_cond = 0;                                                 //   
                TCNT0=0x00;
                }
        if (tick >= 22 && tick < 116 && start_cond)                             //   22  115 
                {
                ++b_cnt;                                                        // "1"    
                if (b_cnt < 9) addr_1 = (addr_1 << 1) + 1;                      //  -  
                if (b_cnt >= 9 && b_cnt < 17) addr_0 = (addr_0 << 1) + 1;       //  -  
                if (b_cnt >= 16 && b_cnt < 25) cmd_1 = (cmd_1 << 1) + 1;        //  -  
                if (b_cnt >= 24) cmd_0 = (cmd_0 << 1) + 1;                      //  -  
                }
        if (tick >= 10 && tick < 22 && start_cond)                              //   10  21 
                {
                ++b_cnt;                                                        // "0"    
                if (b_cnt < 9) addr_1 = (addr_1 << 1);                          // -   
                if (b_cnt >= 9 && b_cnt < 17) addr_0 = (addr_0 << 1);
                if (b_cnt >= 16 && b_cnt < 25) cmd_1 = (cmd_1 << 1);
                if (b_cnt >= 24) cmd_0 = (cmd_0 << 1);
                }
        tick = 0;                                                               // 
        if (b_cnt == 32)                                                        //   4 
                {
//                 if ((addr_1+addr_0) == 0xFF) addr = addr_0;                  //,     First
//                         else addr = 0;                                       //16-bit  0x6120, ..    
                if ((cmd_1 + cmd_0) == 0xFF) cmd = cmd_1;                       //   
                        else cmd = 0;        
                b_cnt = 0;                                                      //  
                start_cond = 0;                                                 //  
                repeat_cond = 0;
                TCCR0=0x00;                                                     // 
                TCNT0=0x00;
                }
        }



// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
        {
        TCNT0=0xFD;                                                             //  -  96
        if (++tick > 1200)                                                      //   1200 
                {
                TCCR0=0x00;                                                     //    
                TCNT0=0x00;
                tick = 0;                                                       
                start_cond = 0;
                repeat_cond = 0;
                //addr_1 = addr_0 = cmd = 0;
                }
        }


void __sign (char digit)        //
        {
        switch (digit)
                {
                case 0 : PORTB = 0b01110111; break;
                
                case 1 : PORTB = 0b00000101; break;
                
                case 2 : PORTB = 0b01111001; break;
                
                case 3 : PORTB = 0b00111101; break;
                
                case 4 : PORTB = 0b00001111; break;
                
                case 5 : PORTB = 0b00111110; break;
                
                case 6 : PORTB = 0b01111110; break;
                
                case 7 : PORTB = 0b00010101; break;
                
                case 8 : PORTB = 0b01111111; break;
                
                case 9 : PORTB = 0b00111111; break;
                
                case 10 : PORTB = 0b01011111; break;   //'a'
                
                case 11 : PORTB = 0b01101110; break;   //'b'
                
                case 12 : PORTB = 0b01110010; break;   //'c'
                
                case 13 : PORTB = 0b01101101; break;   //'d'
                          
                case 14 : PORTB = 0b01111010; break;   //'e'
                          
                case 15 : PORTB = 0b01011010; break;   //'f'
                          
                case 16 : PORTB = 0b00000000; break;   //' '
                
                default:  PORTB = 0b00111000; 
                }
       }

 void __print (char a, char b, char c, char d, char e, char f, int t)      //       a, b,   d   t 
        {
        int i = 0;
        //if (a == 0) a = 20;     //    0 -   
        while (i < t/12)
                {  
                __sign (a);
                PORTC.7 = 1;
                delay_ms(2);
                PORTC.7 = 0;
                
                __sign (b);
                PORTC.6 = 1;
                delay_ms(2);
                PORTC.6 = 0;
                
                __sign (c);
                PORTC.5 = 1;
                delay_ms(2);
                PORTC.5 = 0; 
                
                __sign (d);
                PORTC.2 = 1;
                delay_ms(2); 
                PORTC.2 = 0;
                
                __sign (e);
                PORTC.3 = 1;
                delay_ms(2);
                PORTC.3 = 0;
                
                __sign (f);
                PORTC.4 = 1;
                delay_ms(2);
                PORTC.4 = 0;
                
                i++;
                }
        }       

void main(void)
{
// Declare your local variables here

// Input/Output Ports initialization
// Port A initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=Out Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=P State1=0 State0=P 
PORTA=0x05;
DDRA=0x02;

// Port B initialization
// Func7=Out Func6=Out Func5=Out Func4=Out Func3=Out Func2=Out Func1=Out Func0=Out 
// State7=0 State6=0 State5=0 State4=0 State3=0 State2=0 State1=0 State0=0 
PORTB=0x00;
DDRB=0xFF;

// Port C initialization
// Func7=Out Func6=Out Func5=Out Func4=Out Func3=Out Func2=Out Func1=Out Func0=Out 
// State7=0 State6=0 State5=0 State4=0 State3=0 State2=0 State1=0 State0=0 
PORTC=0x00;
DDRC=0xFF;

// Port D initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=P State1=T State0=T 
PORTD=0x04;
DDRD=0x00;

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 31,250 kHz
// Mode: Normal top=FFh
// OC0 output: Disconnected
TCCR0=0x00;
TCNT0=0x00;
OCR0=0x00;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
// INT2: Off
MCUCR=0x00;
MCUCSR=0x00;

// External Interrupt(s) initialization
// INT0: On
// INT0 Mode: Falling Edge
// INT1: Off
// INT2: Off
GICR|=0x40;
MCUCR=0x02;
MCUCSR=0x00;
GIFR=0x40;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x01;


// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
SFIOR=0x00;

#asm ("sei")

while (1) __print (start_cond, repeat_cond, addr_1/16, addr_1%16, cmd/16, cmd%16, 12);

}
